import json

from axipy import CoordSystem, Geometry, Feature, Attribute, Schema, provider_manager

from PkkInfo.ui.tools.http_helper import HttpClientPkk


def getObjFromJson(obj_js,name):
    try:
        data_obj=obj_js[name]
        return data_obj
    except:
        return None
class StructData:
    def __init__(self):
        self.__att_struct= {}
        self.__fetures=[]
        self.__def_cs=CoordSystem.from_epsg(3857)
    def add(self):
        pass
    def __propertiesInAtt(self,js_properties):
        out_values=[]
        for key in js_properties.keys():
            if key=='options':
                out_values.extend(self.__propertiesInAtt(js_properties[key]))
            out_values.append({key:js_properties[key]})
        return out_values
    def __addStructAtt(self,list_att_ft):
        for att in list_att_ft['att']:
            key=list(att.keys())[0]
            if key in self.__att_struct:
                continue
            value=att[key]
            if value is None or isinstance(value,str):
                self.__att_struct[key]='string'

                continue
            if isinstance(value,int):
                self.__att_struct[key] = 'integer'
                continue
            if isinstance(value, float):
                self.__att_struct[key]='double'
                continue
    def __prepareGeometry(self,js_geometry):
        str_geo_json=json.dumps(js_geometry)
        axi_geo=Geometry.from_geojson(str_geo_json)
        if axi_geo.coordsystem is None:
            axi_geo.coordsystem=self.__def_cs
        return axi_geo
    def __axiFeature(self,virt_feature):
        att={}
        for ft_att in virt_feature['att']:
            name=list(ft_att.keys())[0]
            value=ft_att[name]
            if value is None:
                continue
            att[name]=value
        ft=Feature(att,geometry=virt_feature['geometry'])
        return ft
    def prepareObj(self,json_obj):
        data=getObjFromJson(json_obj,'data')
        if data is None:
            return False
        type=getObjFromJson(data,'type')
        if type is None:
            return False
        if type.lower()!='featurecollection':
            return None
        source_features=getObjFromJson(data,'features')

        for ft in source_features:
            keys_ft=ft.keys()
            virtal_ft={'att':[],'geometry':None}
            for key in keys_ft:
                value_ft=ft[key]
                if key=='geometry':
                    virtal_ft['geometry']=self.__prepareGeometry(value_ft)
                else:
                    if key == 'type':
                        continue

                    if key=='properties':
                        virtal_ft['att'].extend(self.__propertiesInAtt(ft[key]))
                        continue
                    virtal_ft['att'].append({key: ft[key]})
            self.__addStructAtt(virtal_ft)
            self.__fetures.append(self.__axiFeature(virtal_ft))
        return len(self.__fetures)

    def getSchema(self):
        attrs=[]
        for att_key in self.__att_struct.keys():
            type=self.__att_struct[att_key]
            if type=='string':
                attrs.append(Attribute.string(att_key,255))
                continue
            if type=='double':
                attrs.append(Attribute.double(att_key))
                continue
            if type=='integer':
                attrs.append(Attribute.integer(att_key))
                continue
        return Schema(*attrs, coordsystem=self.__def_cs)
    def getFeatures(self):
        return self.__fetures
class PkkInfoTools:
    def __init__(self,call_back=None):
        self.__call_back=call_back
        self.__vecor_table=None
        self.__http_client=HttpClientPkk()
    def findPkk(self,pkk_name):
        json_data=self.__http_client.getObject(pkk_name)
        self.__vecor_table=None
        if json_data is None:
            return 0
        cls_jsonData=StructData()
        count_fts=cls_jsonData.prepareObj(json_data)
        if count_fts==0:
            return 0
        #if self.__vecor_table is None:
        self.__name_mem = "PKK_поиск_"+pkk_name
        definition = {
                'src': '',
                'dataobject': self.__name_mem,
                'schema': cls_jsonData.getSchema()}
        self.__vecor_table = provider_manager.create(definition)
        self.__vecor_table.insert(cls_jsonData.getFeatures())
        return count_fts
    @property
    def tablePkk(self):
        return self.__vecor_table


